--
-- MenuAppAppDelegate.applescript
-- MenuApp
--
-- Created by Jonathan Nathan, JNSoftware LLC on 1/12/11.
-- Copyright 2011 JNSoftware LLC. All rights reserved.
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy of this 
-- software and associated documentation files (the "Software"), to deal in the Software 
-- without restriction, including without limitation the rights to use, copy, modify, 
-- merge, publish, distribute, sublicense, and/or sell copies of the Software, and to 
-- permit persons to whom the Software is furnished to do so, subject to the following conditions:
--
-- The above copyright notice and this permission notice shall be included in all copies or 
-- substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
-- BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
-- NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
-- DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


script MenuAppAppDelegate
	--Classes
	property parent : class "NSObject"
	property NSMenu : class "NSMenu"
	property NSMenuItem : class "NSMenuItem"
	property NSImage : class "NSImage"
	
	--Objects
	property standardUserDefaults : missing value
	property statusMenu : missing value
	property dynamicMenu : missing value
	property statusItemController : missing value
	
	--IBOutlets
	property mainWindow : missing value
	property menuTitleTextField : missing value
	property iconImageView : missing value
	property titleDisplayMatrix : missing value
	property dynamicMenuItemCountTextField : missing value
	property dynamicMenuActionPopUpButton : missing value
	
	--Other
	property defaultsRegistered : false
	property script_version : "1.4.1"
	property script_url : "http://www.jonn8.com"
	property script_email : "mailto:JNSoftware%20LLC%20%3Cinfo%40jonn8.com%3E?subject=MenuApp%20(v" & script_version & ")"
	property script_contribution_url : "https://order.kagi.com/?ELJ"
	property word_to_define : "AppleScript"
	
	--set the initial defaults
	--these are used here and in the statusItemController
	--we could just pass the values to the statusItemController object
	--or use bindings and observe the changes, lots of options...
	on registerDefaults()
		if (not defaultsRegistered) then
			set standardUserDefaults to current application's class "NSUserDefaults"'s standardUserDefaults
			set defaults to {title_display:1, menu_title:"MenuApp", icon_path:"", word_to_define:word_to_define}
			standardUserDefaults's registerDefaults_(defaults)
			set defaultsRegistered to true
		end if
	end registerDefaults
	
	--setup the initial UI (again, this could be done with bindings to eliminate lots of code)
	on awakeFromNib()
		my registerDefaults()
		menuTitleTextField's setStringValue_(standardUserDefaults's objectForKey_("menu_title"))
		set word_to_define to standardUserDefaults's objectForKey_("word_to_define")
		set icon_path to standardUserDefaults's objectForKey_("icon_path")
		if (icon_path ≠ "") then
			set theImage to (my NSImage's alloc)'s initWithContentsOfFile_(icon_path)
			if (theImage ≠ missing value) then
				iconImageView's setImage_(theImage)
				theImage's release()
			end if
		end if
		titleDisplayMatrix's selectCellAtRow_column_((standardUserDefaults's integerForKey_("title_display")) - 1, 0)
	end awakeFromNib
	
	on applicationWillFinishLaunching_(notification)
		--create the initial status menu
		set statusMenu to (my NSMenu's alloc)'s initWithTitle_("statusMenu")
		set menuItem to (my NSMenuItem's alloc)'s init
		menuItem's setTitle_("About MenuApp")
		menuItem's setTarget_(me)
		menuItem's setAction_("about:")
		menuItem's setEnabled_(true)
		statusMenu's addItem_(menuItem)
		menuItem's release()
		statusMenu's addItem_(my NSMenuItem's separatorItem)
		
		set menuItem to (my NSMenuItem's alloc)'s init
		menuItem's setTitle_("JNSoftware Website…")
		menuItem's setTarget_(me)
		menuItem's setAction_("gotoWebsite:")
		menuItem's setEnabled_(true)
		statusMenu's addItem_(menuItem)
		menuItem's release()
		set menuItem to (my NSMenuItem's alloc)'s init
		menuItem's setTitle_("Email Support…")
		menuItem's setTarget_(me)
		menuItem's setAction_("emailSupport:")
		menuItem's setEnabled_(true)
		statusMenu's addItem_(menuItem)
		menuItem's release()
		set menuItem to (my NSMenuItem's alloc)'s init
		menuItem's setTitle_("Contribute…")
		menuItem's setTarget_(me)
		menuItem's setAction_("contribute:")
		menuItem's setEnabled_(true)
		statusMenu's addItem_(menuItem)
		menuItem's release()
		statusMenu's addItem_(my NSMenuItem's separatorItem)
		
		set menuItem to (my NSMenuItem's alloc)'s init
		menuItem's setTitle_("Show Main Window")
		menuItem's setTarget_(me)
		menuItem's setAction_("showMainWindow:")
		menuItem's setEnabled_(true)
		statusMenu's addItem_(menuItem)
		menuItem's release()
		statusMenu's addItem_(my NSMenuItem's separatorItem)
		
		--add the dynamic menu
		set dynamicMenu to (my NSMenu's alloc)'s initWithTitle_("Dynamic Submenu")
		repeat with i from 1 to 3
			set menuItem to (my NSMenuItem's alloc)'s init
			menuItem's setTag_(i)
			menuItem's setTitle_("Menu Item " & i)
			menuItem's setTarget_(me)
			menuItem's setAction_("doSomething:")
			menuItem's setEnabled_(true)
			dynamicMenu's addItem_(menuItem)
			menuItem's release()
		end repeat
		set menuItem to (my NSMenuItem's alloc)'s init
		menuItem's setTitle_("Dynamic Submenu")
		menuItem's setSubmenu_(dynamicMenu)
		statusMenu's addItem_(menuItem)
		menuItem's release()
		dynamicMenu's release()
		statusMenu's addItem_(my NSMenuItem's separatorItem)
		
		--let's add something useful:
		set menuItem to (my NSMenuItem's alloc)'s init
		menuItem's setTitle_("Get Definition…")
		menuItem's setTarget_(me)
		menuItem's setAction_("getDefinition:")
		menuItem's setEnabled_(true)
		statusMenu's addItem_(menuItem)
		menuItem's release()
		statusMenu's addItem_(my NSMenuItem's separatorItem)
		
		set menuItem to (my NSMenuItem's alloc)'s init
		menuItem's setTitle_("Quit MenuApp")
		menuItem's setTarget_(me)
		menuItem's setAction_("quit:")
		menuItem's setKeyEquivalent_("q")
		menuItem's setEnabled_(true)
		statusMenu's addItem_(menuItem)
		menuItem's release()
		
		--instantiate the statusItemController object and set it to use the statusMenu we just created
		set statusItemController to (current application's class "JNS_StatusItemController"'s alloc)'s init
		statusItemController's createStatusItemWithMenu_(statusMenu)
		statusMenu's release()
		
		--open the main window (center it if it's never been opened before)
		if ((standardUserDefaults's objectForKey_("NSWindow Frame main")) = missing value) then mainWindow's |center|
		mainWindow's makeKeyAndOrderFront_(null)
	end applicationWillFinishLaunching_
	
	on applicationShouldTerminate_(sender)
		return current application's NSTerminateNow
	end applicationShouldTerminate_
	
	on applicationShouldTerminateAfterLastWindowClosed_(sender)
		return false
	end applicationShouldTerminateAfterLastWindowClosed_
	
	on applicationWillTerminate_(notification)
		my plead()
		statusItemController's releaseStatusItem()
		statusItemController's release()
	end applicationWillTerminate_
	
	on doSomething_(sender)
		set {tag_, title_} to {sender's |tag|, sender's |title|}
		tell application "System Events"
			activate
			display alert "doSomething" message ("Selection: " & title_ & " (" & tag_ & ")") as informational buttons {"OK"} default button 1 giving up after 10
		end tell
	end doSomething_
	
	on doSomethingElse_(sender)
		set {tag_, title_} to {sender's |tag|, sender's |title|}
		tell application "System Events"
			activate
			display alert "doSomethingElse" message ("Selection: " & title_ & " (" & tag_ & ")") as informational buttons {"OK"} default button 1 giving up after 10
		end tell
	end doSomethingElse_
	
	on updateMenuTitle_(sender)
		set menu_title to sender's stringValue
		standardUserDefaults's setObject_forKey_(menu_title, "menu_title")
		statusItemController's updateDisplay()
	end updateMenuTitle_
	
	on chooseIconPath_(sender)
		try
			set icon_path to POSIX path of (choose file with prompt "Choose an Image for the Icon:")
			set theImage to (my NSImage's alloc)'s initWithContentsOfFile_(icon_path)
			if (theImage ≠ missing value) then
				iconImageView's setImage_(theImage)
				theImage's release()
			end if
		on error e number n
			if n = -128 then --user cancelled
				set icon_path to ""
				set theImage to (my NSImage's imageNamed_("NSActionTemplate"))
				iconImageView's setImage_(theImage)
			else
				log "Error: " & e
				return
			end if
		end try
		standardUserDefaults's setObject_forKey_(icon_path, "icon_path")
		statusItemController's updateDisplay()
	end chooseIconPath_
	
	on updateMenuTitleDisplay_(sender)
		set title_display to ((sender's selectedRow) as integer) + 1
		standardUserDefaults's setInteger_forKey_(title_display, "title_display")
		statusItemController's updateDisplay()
	end updateMenuTitleDisplay_
	
	on updateMenuAnimationDisplay_(sender)
		set animation_display to ((sender's selectedRow) as integer) + 1
		statusItemController's updateAnimation_(animation_display)
	end updateMenuAnimationDisplay_
	
	on reloadMenuNow_(sender)
		statusItemController's updateAnimation_(3)
		dynamicMenu's removeAllItems()
		set item_count to (dynamicMenuItemCountTextField's integerValue) as integer
		if (item_count = 0) then
			set menuItem to (my NSMenuItem's alloc)'s init
			menuItem's setTitle_("Menu Item Count Set to 0")
			menuItem's setEnabled_(false)
			dynamicMenu's addItem_(menuItem)
			menuItem's release()
		else
			set the_action to ((dynamicMenuActionPopUpButton's titleOfSelectedItem) as Unicode text) & ":"
			set current_date to (current date)'s time string
			repeat with i from 1 to item_count
				set menuItem to (my NSMenuItem's alloc)'s init
				menuItem's setTag_(i)
				menuItem's setTitle_("Reloaded Menu Item " & i & " (" & current_date & ")")
				menuItem's setTarget_(me)
				menuItem's setAction_(the_action)
				menuItem's setEnabled_(true)
				dynamicMenu's addItem_(menuItem)
				menuItem's release()
			end repeat
		end if
		statusItemController's updateAnimation_(1)
	end reloadMenuNow_
	
	on about_(sender)
		activate
		current application's NSApp's orderFrontStandardAboutPanel_(null)
	end about_
	
	on showMainWindow_(sender)
		activate
		mainWindow's makeKeyAndOrderFront_(null)
	end showMainWindow_
	
	on gotoWebsite_(sender)
		open location script_url
	end gotoWebsite_
	
	on emailSupport_(sender)
		open location script_email
	end emailSupport_
	
	on contribute_(sender)
		open location script_contribution_url
	end contribute_
	
	on quit_(sender)
		quit
	end quit_
	
	on plead()
		tell application "System Events"
			activate
			set the_button to button returned of (display dialog "If you found this useful, and we hope you did, please consider contributing to our web hosting and general development costs. (You may disable this reminder in the source code.)" buttons {"Maybe Later", "Contribute Now"} default button 2 with icon 1 with title "MenuApp")
		end tell
		if the_button = "Contribute Now" then open location script_contribution_url
	end plead
	
	--try searching for your name or a ZIP code, anything, really
	on getDefinition_(sender)
		repeat
			tell application "System Events"
				activate
				try
					set word_to_define to text returned of (display dialog "Get definition for:" default answer word_to_define as Unicode text buttons {"Cancel", "OK"} default button 2 with icon 1 with title "MenuApp Get Definition")
				on error e number n
					if n = -128 then return --user cancelled
				end try
			end tell
			standardUserDefaults's setObject_forKey_(word_to_define, "word_to_define")
			set the_definitions to my get_definition(word_to_define)
			set definition_count to (count the_definitions)
			if definition_count = 1 then
				set the_definition to (the_definitions's item 1)
			else
				set the_sources to {}
				repeat with i from 1 to definition_count
					set end of the_sources to (the_definitions's item i)'s paragraph 1
				end repeat
				tell application "System Events"
					activate
					set the_choice to (choose from list the_sources with prompt ((definition_count as string) & " definitions were found for “" & word_to_define & ".” Here are the sources of the definitions:") OK button name "Show Definition" with title "Multiple Definitions Found" default items {the_sources's item 1} without multiple selections allowed and empty selection allowed) as string
				end tell
				if the_choice = "false" then return
				repeat with i from 1 to definition_count
					if (the_choice = (the_sources's item i)) then
						set the_definition to (item i of the_definitions)
						exit repeat
					end if
				end repeat
			end if
			set the_button to my display_definition(the_definition)
			if the_button = "Copy to Clipboard" then
				set the clipboard to the_definition
				exit repeat
			else if the_button = "OK" then
				exit repeat
			end if
		end repeat
		return the_definition
	end getDefinition_
	
	on get_definition(word_to_define)
		try
			statusItemController's updateAnimation_(3)
			set ASCII_13 to (ASCII character 13)
			set the_script to my snr(("dict://dict.org/d:" & word_to_define & ":*"), " ", "%20")
			--Thanks to Greg Spense for the grep code
			set the_definitions to do shell script "curl -s " & (quoted form of the_script) & " | egrep -v '^220.*|^250.*|^150.*|^221.*'"
			set the_definitions to my snr(the_definitions, "151 ", "")
			set the_definitions to my string_to_list(the_definitions, (ASCII_13 & "." & ASCII_13))
			statusItemController's updateAnimation_(1)
			return the_definitions
		on error the_err
			statusItemController's updateAnimation_(1)
			return the_err
		end try
	end get_definition
	
	on display_definition(the_definition)
		tell application "System Events"
			activate
			return button returned of (display alert ("Definition of “" & word_to_define & "”") message the_definition as informational buttons {"Copy to Clipboard", "Search Again", "OK"} default button 3)
		end tell
	end display_definition
	
	on snr(the_string, search_string, replace_string)
		tell (a reference to AppleScript's text item delimiters)
			set {old_tid, contents} to {contents, search_string}
			set {the_string, contents} to {the_string's text items, replace_string}
			set {the_string, contents} to {the_string as Unicode text, old_tid}
		end tell
		return the_string
	end snr
	
	on string_to_list(s, d)
		tell (a reference to AppleScript's text item delimiters)
			set {o, contents} to {contents, d}
			set {s, contents} to {s's text items, o}
		end tell
		return s
	end string_to_list
	
end script
